package cn.efunbox.manage.base.service.impl;

import cn.efunbox.manage.base.entity.Company;
import cn.efunbox.manage.base.enums.BaseStatusEnum;
import cn.efunbox.manage.base.repository.CompanyRepository;
import cn.efunbox.manage.base.service.CompanyService;
import cn.efunbox.manage.common.enums.BaseOrderEnum;
import cn.efunbox.manage.common.helper.SortHelper;
import cn.efunbox.manage.common.result.ApiCode;
import cn.efunbox.manage.common.result.ApiResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Objects;

/**
 * CompanyServiceImpl
 * Created by wangys on 2019/03/08
 */
@Service
public class CompanyServiceImpl implements CompanyService {

    @Autowired
    CompanyRepository companyRepository;


    @Override
    public ApiResult list(Company company) {
        List<Company> companies = companyRepository.find(company, SortHelper.sortMap2Sort(new LinkedHashMap<String, BaseOrderEnum>(){{
            put("code",BaseOrderEnum.ASC);
        }}));
        if(Objects.nonNull(companies)){
            return ApiResult.ok(companies);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult save(Company company) {
        if(company == null){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        company = companyRepository.save(company);
        if(Objects.nonNull(company)){
            return ApiResult.ok(company);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult update(Company company) {
        if(company == null){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        company = companyRepository.update(company);
        if(Objects.nonNull(company)){
            return ApiResult.ok(company);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult getById(Long companyId) {
        if(Objects.isNull(companyId)){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        Company company = companyRepository.find(companyId);
        if(Objects.nonNull(company)){
            return ApiResult.ok(company);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult select() {
        return ApiResult.ok(companyRepository.findByStatusOrderByCodeAsc(BaseStatusEnum.NORMAL));
    }
}
