package cn.efunbox.manage.base.service.impl;

import cn.efunbox.manage.base.entity.CostType;
import cn.efunbox.manage.base.enums.BaseStatusEnum;
import cn.efunbox.manage.base.repository.CostTypeRepository;
import cn.efunbox.manage.base.service.CostTypeService;
import cn.efunbox.manage.common.enums.BaseOrderEnum;
import cn.efunbox.manage.common.helper.SortHelper;
import cn.efunbox.manage.common.result.ApiCode;
import cn.efunbox.manage.common.result.ApiResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Objects;

/**
 * CostTypeServiceImpl
 * Created by wangys on 2019/03/08
 */
@Service
public class CostTypeServiceImpl implements CostTypeService {

    @Autowired
    CostTypeRepository costTypeRepository;

    @Override
    public ApiResult list(CostType costType) {
        List<CostType> costTypes = costTypeRepository.find(costType, SortHelper.sortMap2Sort(new LinkedHashMap<String,BaseOrderEnum>() {{
            put("code",BaseOrderEnum.ASC);
        }}));
        if(Objects.nonNull(costTypes)){
            return ApiResult.ok(costTypes);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult save(CostType costType) {
        if(costType == null){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        if(Objects.nonNull(costTypeRepository.findByName(costType.getName()))){
            return ApiResult.error(ApiCode.REPETITION_ERROR);
        }
        costType = costTypeRepository.save(costType);
        if(Objects.nonNull(costType)){
            return ApiResult.ok(costType);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult update(CostType costType) {
        if(costType == null){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        CostType oldCostType = costTypeRepository.findByName(costType.getName());
        if(Objects.nonNull(oldCostType) && !Objects.equals(oldCostType.getId(), costType.getId())){
            return ApiResult.error(ApiCode.REPETITION_ERROR);
        }
        costType = costTypeRepository.update(costType);
        if(Objects.nonNull(costType)){
            return ApiResult.ok(costType);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult getById(Long costTypeId) {
        if(Objects.isNull(costTypeId)){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        CostType costType = costTypeRepository.find(costTypeId);
        if(costType ==null){
            return ApiResult.error(ApiCode.NOT_FOUND);
        }
        return ApiResult.ok(costType);
    }

    @Override
    public ApiResult select(CostType costType) {
        return ApiResult.ok(costTypeRepository.findByStatusOrderByCodeAsc(BaseStatusEnum.NORMAL));
    }
}
