package cn.efunbox.manage.base.service.impl;

import cn.efunbox.manage.base.entity.*;
import cn.efunbox.manage.base.enums.BaseStatusEnum;
import cn.efunbox.manage.base.enums.IncomeStatusEnum;
import cn.efunbox.manage.base.enums.ManageTypeEnum;
import cn.efunbox.manage.base.repository.*;
import cn.efunbox.manage.base.service.ManageService;
import cn.efunbox.manage.base.vo.ManageVO;
import cn.efunbox.manage.common.enums.BaseOrderEnum;
import cn.efunbox.manage.common.helper.SortHelper;
import cn.efunbox.manage.common.result.ApiCode;
import cn.efunbox.manage.common.result.ApiResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.Predicate;
import java.math.BigDecimal;
import java.util.*;

/**
 * ManageSreviceImpl
 * Created by wangys on 2019/03/08
 */
@Service
public class ManageServiceImpl implements ManageService {

    @Autowired
    ManageRepository manageRepository;

    @Autowired
    DepartmentRepository departmentRepository;

    @Autowired
    ProjectRepository projectRepository;

    @Autowired
    ProductRepository productRepository;

    @Autowired
    CompanyRepository companyRepository;

    @Autowired
    CostTypeRepository costTypeRepository;

    @Override
    public ApiResult list(ManageVO manageVO) {
        Pageable pageable = new PageRequest(manageVO.getPageNo() - 1, manageVO.getPageSize(), SortHelper.sortMap2Sort(new LinkedHashMap<String, BaseOrderEnum>(){{put("status", BaseOrderEnum.ASC);put("gmtModified", BaseOrderEnum.DESC);}}));
//        manageVO.setStatus(BaseStatusEnum.NORMAL);
        if(!manageVO.getIsEntry()){
            pageable = new PageRequest(manageVO.getPageNo() - 1,manageVO.getPageSize(), SortHelper.sortMap2Sort(new LinkedHashMap<String, BaseOrderEnum>(){{put("status", BaseOrderEnum.ASC);put("happenTime", BaseOrderEnum.DESC);}}));
            manageVO.setStatus(BaseStatusEnum.NORMAL);
        }
        Specification specification = (Specification<Manage>) (root, criteriaQuery, criteriaBuilder) -> {

            List<Predicate> predicates = new ArrayList<>();
            if(manageVO.getToProjectId() != null && manageVO.getToProjectId().size() > 0){
                List<Predicate> list = new ArrayList<>();
                if (manageVO.getToProjectId() != null && manageVO.getToProjectId().size() > 0) {
                    CriteriaBuilder.In<Object> in = criteriaBuilder.in(root.get("toProjectId"));
                    for (Long id : manageVO.getToProjectId()) {
                        in.value(id);
                    }
                    list.add(in);
                }
                predicates.addAll(list);
            }
//            if(Objects.nonNull(manageVO.getToProjectId()) && 0 != manageVO.getToProjectId()){
//                predicates.add(criteriaBuilder.equal(root.get("toProjectId"), manageVO.getToProjectId()));
//            }
            if(manageVO.getToDeptId() != null && manageVO.getToDeptId().size() > 0){
                List<Predicate> list = new ArrayList<>();
                if (manageVO.getToDeptId() != null && manageVO.getToDeptId().size() > 0) {
                    CriteriaBuilder.In<Object> in = criteriaBuilder.in(root.get("toDeptId"));
                    for (Long id : manageVO.getToDeptId()) {
                        in.value(id);
                    }
                    list.add(in);
                }
                predicates.addAll(list);
            }
//            if(Objects.nonNull(manageVO.getToDeptId()) && 0 != manageVO.getToDeptId()){
//                predicates.add(criteriaBuilder.equal(root.get("toDeptId"), manageVO.getToDeptId()));
//            }
            if(manageVO.getProductId() != null && manageVO.getProductId().size() > 0){
                List<Predicate> list = new ArrayList<>();
                if (manageVO.getProductId() != null && manageVO.getProductId().size() > 0) {
                    CriteriaBuilder.In<Object> in = criteriaBuilder.in(root.get("productId"));
                    for (Long id : manageVO.getProductId()) {
                        in.value(id);
                    }
                    list.add(in);
                }
                predicates.addAll(list);
            }
//            if(Objects.nonNull(manageVO.getProductId()) && 0 != manageVO.getProductId()){
//                predicates.add(criteriaBuilder.equal(root.get("productId"), manageVO.getProductId()));
//            }
            if(manageVO.getProjectId() != null && manageVO.getProjectId().size() > 0){
                List<Predicate> list = new ArrayList<>();
                if (manageVO.getProjectId() != null && manageVO.getProjectId().size() > 0) {
                    CriteriaBuilder.In<Object> in = criteriaBuilder.in(root.get("projectId"));
                    for (Long id : manageVO.getProjectId()) {
                        in.value(id);
                    }
                    list.add(in);
                }
                predicates.addAll(list);
            }
//            if(Objects.nonNull(manageVO.getProjectId()) && 0 != manageVO.getProjectId()){
//                predicates.add(criteriaBuilder.equal(root.get("projectId"), manageVO.getProjectId()));
//            }
            if(Objects.nonNull(manageVO.getType())){
                predicates.add(criteriaBuilder.equal(root.get("type"), manageVO.getType()));
            }
            if(manageVO.getCostTypeId() != null && manageVO.getCostTypeId().size() > 0){
                List<Predicate> list = new ArrayList<>();
                if (manageVO.getCostTypeId() != null && manageVO.getCostTypeId().size() > 0) {
                    CriteriaBuilder.In<Object> in = criteriaBuilder.in(root.get("costTypeId"));
                    for (Long id : manageVO.getCostTypeId()) {
                        in.value(id);
                    }
                    list.add(in);
                }
                predicates.addAll(list);
            }
//                if(Objects.nonNull(manageVO.getCostTypeId()) && 0 != manageVO.getCostTypeId()){
//                    predicates.add(criteriaBuilder.equal(root.get("costTypeId"), manageVO.getCostTypeId()));
//                }
            if(manageVO.getIncomeStatus() != null && manageVO.getIncomeStatus().size() > 0){
                List<Predicate> list = new ArrayList<>();
                if (manageVO.getIncomeStatus() != null && manageVO.getIncomeStatus().size() > 0) {
                    CriteriaBuilder.In<Object> in = criteriaBuilder.in(root.get("incomeStatus"));
                    for (IncomeStatusEnum incomeStatus : manageVO.getIncomeStatus()) {
                        in.value(incomeStatus);
                    }
                    list.add(in);
                }
                predicates.addAll(list);
            }
//                if(Objects.nonNull(manageVO.getIncomeStatus())){
//                    predicates.add(criteriaBuilder.equal(root.get("incomeStatus"), manageVO.getIncomeStatus()));
//                }

            if(Objects.nonNull(manageVO.getStatus())){
                predicates.add(criteriaBuilder.equal(root.get("status"), manageVO.getStatus()));
            }
            if(Objects.nonNull(manageVO.getStartTime())){
                predicates.add(criteriaBuilder.greaterThanOrEqualTo(root.get("happenTime"), manageVO.getStartTime()));
            }
            if(Objects.nonNull(manageVO.getEndTime())){
                predicates.add(criteriaBuilder.lessThanOrEqualTo(root.get("happenTime"), manageVO.getEndTime()));
            }
            if(Objects.nonNull(manageVO.getMinAmount()) && manageVO.getMinAmount() > 0){
                predicates.add(criteriaBuilder.ge(root.get("amount"), manageVO.getMinAmount()));
            }
            if(Objects.nonNull(manageVO.getMaxAmount()) && manageVO.getMaxAmount() > 0){
                predicates.add(criteriaBuilder.le(root.get("amount"), manageVO.getMaxAmount()));
            }
            if(manageVO.getDeptIds() != null && manageVO.getDeptIds().size() > 0){
                List<Predicate> list = new ArrayList<>();
                if (manageVO.getDeptIds() != null && manageVO.getDeptIds().size() > 0) {
                    CriteriaBuilder.In<Object> in = criteriaBuilder.in(root.get("deptId"));
                    for (Long id : manageVO.getDeptIds()) {
                        in.value(id);
                    }
                    list.add(in);
                }
                predicates.addAll(list);
            }
            return criteriaBuilder.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        Long count = manageRepository.count(specification);
        if (count == 0) {
            return ApiResult.ok(null);
        }
        Page<Manage> manages = manageRepository.findAll(specification, pageable);
        List<Manage> totalManages =  manageRepository.findAll(specification);
        BigDecimal sum = new BigDecimal("0");
        for (Manage manage: totalManages) {
            if (Objects.equals(manage.getType(), ManageTypeEnum.COST)){
                sum = sum.subtract(new BigDecimal(Double.toString(manage.getAmount())));
            }else {
                sum = sum.add(new BigDecimal(Double.toString(manage.getAmount())));
            }
        }
        if(Objects.isNull(manages)){
            return ApiResult.error(ApiCode.UNKNOWN_ERROR);

        }
        List<Long> deptIds = new ArrayList<>();
        List<Long> projectIds = new ArrayList<>();
        List<Long> productIds = new ArrayList<>();
        List<Long> companyIds = new ArrayList<>();
        List<Long> costTypeIds = new ArrayList<>();
        for(Manage m : manages){
            m.setTotalAmount(sum.doubleValue());
            if(Objects.nonNull(m.getDeptId())){
                deptIds.add(m.getDeptId());
            }
            if(Objects.nonNull(m.getProjectId())){
                projectIds.add(m.getProjectId());
            }
            if(Objects.nonNull(m.getToDeptId())){
                deptIds.add(m.getToDeptId());
            }
//            if(Objects.equals(m.getType(), ManageTypeEnum.COST)){
//                switch (m.getCompanyType()){
//                    case COMPANY:
//                        if(Objects.nonNull(m.getCompanyId())){
//                            companyIds.add(m.getCompanyId());
//                        }
//                        break;
//                    case DEPARTMENT:
//                        if(Objects.nonNull(m.getCompanyId())){
//                            deptIds.add(m.getCompanyId());
//                        }
//                        break;
//                    default:
//                        break;
//                }
//            }else {
//                if(Objects.nonNull(m.getProductId())){
//                    productIds.add(m.getProductId());
//                }
//            }
            if(Objects.nonNull(m.getProductId())){
                productIds.add(m.getProductId());
            }
            if(Objects.nonNull(m.getCostTypeId())){
                costTypeIds.add(m.getCostTypeId());
            }
            if(Objects.nonNull(m.getToProjectId())){
                projectIds.add(m.getToProjectId());
            }
            if(Objects.nonNull(m.getToDeptId())){
                deptIds.add(m.getToDeptId());
            }
        }

        //部门
        List<Department> departments = departmentRepository.findByIds(deptIds);
        Map<Long, Department> departmentMap = new HashMap<>();
        departments.stream().forEach(department -> {
            departmentMap.put(department.getId(), department);
        });
        //项目
        List<Project> projects = projectRepository.findByIds(projectIds);
        Map<Long, Project> projectMap = new HashMap<>();
        projects.stream().forEach(project -> {
            projectMap.put(project.getId(), project);
        });
        //产品
        List<Product> products = productRepository.findByIds(productIds);
        Map<Long, Product> producMap = new HashMap<>();
        products.stream().forEach(product -> {
            producMap.put(product.getId(), product);
        });
        //支出发起人
        List<Company> companies = companyRepository.findByIds(companyIds);
        Map<Long, Company> companyMap = new HashMap<>();
        companies.stream().forEach(company -> {
            companyMap.put(company.getId(), company);
        });
        //成本类型
        List<CostType> costTypes = costTypeRepository.findByIds(costTypeIds);
        Map<Long, CostType> costTypeMap = new HashMap<>();
        costTypes.stream().forEach(costType -> {
            costTypeMap.put(costType.getId(), costType);
        });

        for(Manage m : manages){
            m.setDepartment(departmentMap.get(m.getDeptId()));
            m.setProject(projectMap.get(m.getProjectId()));
            m.setProduct(producMap.get(m.getProductId()));
            m.setToProject(projectMap.get(m.getToProjectId()));
            if(Objects.nonNull(m.getCostTypeId())){
                m.setCostType(costTypeMap.get(m.getCostTypeId()));
            }
            m.setToDepartment(departmentMap.get(m.getToDeptId()));
//            if(Objects.nonNull(m.getCompanyType())){
//                switch (m.getCompanyType()){
//                    case COMPANY:
//                        if(Objects.nonNull(m.getCompanyId())){
//                            m.setCompany(companyMap.get(m.getCompanyId()));
//                        }
//                        break;
//                    case DEPARTMENT:
//                        if(Objects.nonNull(m.getCompanyId())){
//                            m.setCompany(companyMap.get(m.getDeptId()));
//                        }
//                        break;
//                    default:
//                        break;
//                }
//            }
        };
        return ApiResult.ok(manages);
    }

    @Override
    public ApiResult save(Manage manage) {
        if(Objects.isNull(manage)){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        manage = manageRepository.save(manage);
        if(Objects.nonNull(manage)){
            return ApiResult.ok(manage);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult update(Manage manage) {
        if(Objects.isNull(manage)){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        manage = manageRepository.update(manage);
        if(Objects.nonNull(manage)){
            return ApiResult.ok(manage);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult getById(Long manageId) {
        if(Objects.isNull(manageId)){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        Manage manage = manageRepository.find(manageId);
        if(Objects.isNull(manage)){
            return ApiResult.error(ApiCode.UNKNOWN_ERROR);
        }
        manage.setDepartment(departmentRepository.find(manage.getDeptId()));
        manage.setProject(projectRepository.find(manage.getProjectId()));
        manage.setProduct(productRepository.find(manage.getProductId()));
        manage.setToProject(projectRepository.find(manage.getToProjectId()));
        manage.setToDepartment(departmentRepository.find(manage.getToDeptId()));
//        if(Objects.nonNull(manage.getCompanyType())){
//
//            switch (manage.getCompanyType()){
//                case COMPANY:
//                    if(Objects.nonNull(manage.getCompanyId())){
//                        manage.setCompany(companyRepository.find(manage.getCompanyId()));
//                    }
//                    break;
//                case DEPARTMENT:
//                    if(Objects.nonNull(manage.getCompanyId())){
//                        manage.setCompany(departmentRepository.find(manage.getDeptId()));
//                    }
//                    break;
//                default:
//                    break;
//            }
//        }
        return ApiResult.ok(manage);
    }
}
