package cn.efunbox.manage.base.service.impl;

import cn.efunbox.manage.base.entity.Project;
import cn.efunbox.manage.base.enums.BaseStatusEnum;
import cn.efunbox.manage.base.repository.DepartmentRepository;
import cn.efunbox.manage.base.repository.ProjectRepository;
import cn.efunbox.manage.base.service.ProjectService;
import cn.efunbox.manage.common.enums.BaseOrderEnum;
import cn.efunbox.manage.common.helper.SortHelper;
import cn.efunbox.manage.common.result.ApiCode;
import cn.efunbox.manage.common.result.ApiResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Objects;

/**
 * ProjectServiceImpl
 * Created by wangys on 2019/03/08
 */
@Service
public class ProjectServiceImpl implements ProjectService {

    @Autowired
    ProjectRepository projectRepository;

    @Autowired
    DepartmentRepository departmentRepository;

    @Override
    public ApiResult list(Project project) {
        List<Project> projects = projectRepository.find(project, SortHelper.sortMap2Sort(new LinkedHashMap<String,BaseOrderEnum>(){{
            put("code",BaseOrderEnum.ASC);
        }}));
        if(Objects.nonNull(projects)){
            return ApiResult.ok(projects);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult save(Project project) {
        if(Objects.isNull(project)){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        if(Objects.nonNull(projectRepository.findByName(project.getName()))){
            return ApiResult.error(ApiCode.REPETITION_ERROR);
        }
        project = projectRepository.save(project);
        if(Objects.nonNull(project)){
            return ApiResult.ok(project);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult update(Project project) {
        if(project ==null){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        Project oldProject = projectRepository.findByName(project.getName());
        if(Objects.nonNull(oldProject) && !Objects.equals(oldProject.getId(), project.getId())){
            return ApiResult.error(ApiCode.REPETITION_ERROR);
        }
        project = projectRepository.update(project);
        if(Objects.nonNull(project)){
            return ApiResult.ok(project);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult getProjectById(Long projectId) {
        if(Objects.isNull(projectId)){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        Project project = projectRepository.find(projectId);
        if(project ==null){
            return ApiResult.error(ApiCode.NOT_FOUND);
        }
        return ApiResult.ok(project);
    }

    @Override
    public ApiResult getSelect() {
        List<Project> projects = projectRepository.findByStatusOrderByCodeAsc(BaseStatusEnum.NORMAL);
        return ApiResult.ok(projects);
    }


}
