package cn.efunbox.manage.base.assist.aspect;

import cn.efunbox.manage.base.assist.async.AsyncOperationLogHandler;
import cn.efunbox.manage.base.entity.User;
import cn.efunbox.manage.base.repository.UserRepository;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.annotation.Order;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;

@Slf4j
@Aspect
@Component
@Order(-100)
public class OperationLogAspect {

    @Autowired
    AsyncOperationLogHandler asyncOperationLogHandler;
    @Autowired
    RedisTemplate redisTemplate;

    @Autowired
    UserRepository userRepository;

    @AfterReturning(value = "execution(* cn.efunbox.manage.base.controller..*.*(..))",returning = "retVal")
    public void doAfter(JoinPoint joinPoint,Object retVal) throws Throwable {
        // 接收到请求，记录请求内容
        try {
            ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
            HttpServletRequest request=attributes.getRequest();
            if(request.getRequestURL().indexOf("/upload")!=-1){
                log.debug("上传文件 忽略!");
                return;
            }

            //如果是 增删改 才记录入库
            if(request.getMethod().equals("POST")|| request.getMethod().equals("PUT")|| request.getMethod().equals("DELETE")|| request.getMethod().equals("TRACE")|| request.getMethod().equals("PATCH")){
//                String token = request.getHeader("X-Token");
                String uid = request.getHeader("uid");
                if (StringUtils.isEmpty(uid)) {
                    log.error("user uid is null !");
                    return;
                }

//                User loginUser = (User) redisTemplate.opsForValue().get(token);
                User loginUser = userRepository.findById(uid);


                request.setAttribute("userInfo",loginUser);

                asyncOperationLogHandler.asyncHandleLog(joinPoint,request,retVal);
            }
        } catch (Exception e) {
            log.error("日志记录出错!", e);
        }
    }

}