package cn.efunbox.manage.common.result;

import java.io.Serializable;
import java.util.Map;
import java.util.Objects;

public class ApiResult<T> implements Serializable {
    private static final long serialVersionUID = 1L;
    private boolean success;
    private int code;
    private int count;
    private String message;
    private T data;

    public ApiResult() {
        this.success = true;
    }

    public ApiResult(ApiCode apiCode) {
        this(apiCode.getCode(),apiCode.getMessage());
    }

    private ApiResult(int code, String message) {
        this.success = true;
        this.code = code;
        if(code != 200) {
            this.success = false;
        }

        this.message = message;
    }

    private ApiResult(ApiCode apiCode, T data) {
        this(apiCode.getCode(), apiCode.getMessage());
        this.data = data;
    }

    private ApiResult(ApiCode apiCode, T data,Integer count) {
        this(apiCode.getCode(), apiCode.getMessage());
        this.data = data;
        this.count = count;
    }

    public static ApiResult ok() {
        return new ApiResult(ApiCode.OK);
    }

    public static ApiResult ok(Object value) {
        return new ApiResult(ApiCode.OK, value);
    }

    public static ApiResult ok(Object value,Integer award) {
        if (Objects.isNull(award)) {
            award = 0;
        }
        return new ApiResult(ApiCode.OK, value,award);
    }


    public static ApiResult ok(Map<String, Object> data) {
        return new ApiResult(ApiCode.OK, data);
    }


    public static ApiResult error(ApiCode code) {
        return new ApiResult(code);
    }

    public static ApiResult error(ApiCode code, Object value) {
        return new ApiResult(code, value);
    }

    public static ApiResult error(ApiCode code, Map<String, Object> data) {
        return new ApiResult(code, data);
    }

    public boolean getSuccess() {
        return this.success;
    }

    public void setSuccess(boolean success) {
        this.success = success;
    }

    public int getCode() {
        return this.code;
    }

    public void setCode(int code) {
        this.code = code;
    }
    public int getCount() {
        return this.count;
    }

    public void setCount(int count) {
        this.count = count;
    }

    public String getMessage() {
        return this.message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public T getData() {
        return this.data;
    }

    public void setData(T data) {
        this.data = data;
    }
}
