package cn.efunbox.manage.common.utils;


import cn.efunbox.manage.common.result.ApiCode;
import cn.efunbox.manage.common.result.ApiResult;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.util.CollectionUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;


/**
 * IaasRequestUtils
 * Created by xusq on 2017/10/19.
 */
@Slf4j
public class IaasRequestUtils {

    private IaasRequestUtils(){}


    public static HttpHeaders map2HttpHeaders(Map<String,String> headerMap){

        ServletRequestAttributes attrs = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (null == attrs) {
            log.error("error operation, is not in RequestContext");
            return null;
        }
        HttpServletRequest request = attrs.getRequest();


        String dist=request.getParameter("dist");
        String appCode=request.getParameter("appCode");
        String appVer=request.getParameter("appver");
        String platform=request.getParameter("platform");
        String os=request.getParameter("os");

        String iaasDist = "0001";

        if (StringUtils.isNotBlank(dist)) {
            iaasDist = dist;
        } else if (StringUtils.isNotBlank(appCode)){
            iaasDist = appCode;
        }

        HttpHeaders httpHeaders = new HttpHeaders();
        MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
        httpHeaders.setContentType(type);
        httpHeaders.add("dist",iaasDist);
        httpHeaders.add("Accept", MediaType.APPLICATION_JSON.toString());
        httpHeaders.add("bizCode", Constants.BIZ_CODE);
        httpHeaders.add("appCode", Constants.APP_CODE);
        if (StringUtils.isNotBlank(appVer)) {
            httpHeaders.add("appVer",appVer);
        } else {
            httpHeaders.add("appVer","2.0");
        }
        if (StringUtils.isNotBlank(os)) {
            httpHeaders.add("os",os);
        } else {
            httpHeaders.add("os","0");
        }if (StringUtils.isNotBlank(platform)) {
            httpHeaders.add("platform",platform);
        } else {
            httpHeaders.add("platform","3");
        }
        httpHeaders.add("userIp", IpUtils.getClientIpAddress(request));
        System.out.println("userIp====="+IpUtils.getClientIpAddress(attrs.getRequest()));
        String test=httpHeaders.getFirst("platform");
        System.out.println("platform======="+test);
        if (CollectionUtils.isEmpty(headerMap)) {
            return httpHeaders;
        }

        for (String key : headerMap.keySet()) {
            httpHeaders.add(key,headerMap.get(key));
        }

        return httpHeaders;
    }

    public static ApiResult checkIaasResult(ResponseEntity<ApiResult> responseEntity){

        HttpStatus statusCode = responseEntity.getStatusCode();
        log.info("request iaas return statusCode : {}",statusCode);
        log.info("request iaas result : {}",JSON.toJSONString(responseEntity.getBody()));
        if (HttpStatus.OK.equals(statusCode)) {

            return responseEntity.getBody();
        } else {
            return ApiResult.error(ApiCode.SERVER_ERROR);
        }
    }

}
